
/*
 *
 * collectables.c - The collectables
 *
 */

#include "common.h"
#include "random.h"
#include "video.h"
#include "input.h"
#include "staticGeometry.h"
#include "collectables.h"

// -----------------------------------------------------------------------
// variables
// -----------------------------------------------------------------------

struct collectable collectables[COLLECTABLES_MAX];

// -----------------------------------------------------------------------
// init the collectables
// -----------------------------------------------------------------------

void collectablesInit(void) {

	int i;

	for (i = 0; i < COLLECTABLES_MAX; i++)
		collectables[i].type = COLLECTABLE_TYPE_NONE;
}

// -----------------------------------------------------------------------
// draw the collectables
// -----------------------------------------------------------------------

void collectablesDraw(void) {

	int i;

	for (i = 0; i < COLLECTABLES_MAX; i++) {
		if (collectables[i].type == COLLECTABLE_TYPE_NONE)
			continue;
		if (collectables[i].type == COLLECTABLE_TYPE_REMOVE)
			continue;

		videoMatrixPush();

		videoTranslate3f(collectables[i].cX, collectables[i].cY, -1550.0f);
		videoRotate3f(0.0f, 0.0f, collectables[i].angle);

		if (collectables[i].type == COLLECTABLE_TYPE_OIL)
			videoDrawMeshTextured(verticesBarrel, verticesBarrelTriangles);

		videoMatrixPop();
	}
}

// -----------------------------------------------------------------------
// remove the specified collectable
// -----------------------------------------------------------------------

void collectablesRemove(float rX, float rY) {

	float d;
	int i;

	for (i = 0; i < COLLECTABLES_MAX; i++) {
		if (collectables[i].type == COLLECTABLE_TYPE_NONE)
			continue;

		d = collectables[i].cX - rX;
		if (d < 0.0f)
			d = -d;
		if (d > 0.01f)
			continue;

		d = collectables[i].cY - rY;
		if (d < 0.0f)
			d = -d;
		if (d > 0.01f)
			continue;

		collectables[i].type = COLLECTABLE_TYPE_NONE;
	}
}

// -----------------------------------------------------------------------
// add a collectable
// -----------------------------------------------------------------------

int  collectablesAdd(int type, float cX, float cY, float angle) {

	int i;

	for (i = 0; i < COLLECTABLES_MAX; i++) {
		if (collectables[i].type == COLLECTABLE_TYPE_NONE) {
			collectables[i].type = type;
			collectables[i].cX = cX;
			collectables[i].cY = cY;
			collectables[i].angle = angle;
			return i;
		}
	}

	return -1;
}
