
/*
 *
 * particles.c - The particles
 *
 */

#include "common.h"
#include "random.h"
#include "video.h"
#include "input.h"
#include "staticGeometry.h"
#include "game.h"
#include "particles.h"

// -----------------------------------------------------------------------
// variables
// -----------------------------------------------------------------------

// the particles
struct particle particles[PARTICLES_MAX];

// the particle life times
int particleLifeTimes[] = {
	-1,   // NONE
	2000, // DUST
	4000, // SMOKE PIPE
};

// -----------------------------------------------------------------------
// init the particles
// -----------------------------------------------------------------------

void particlesInit(void) {

	int i;

	for (i = 0; i < PARTICLES_MAX; i++)
		particles[i].type = PARTICLE_TYPE_NONE;
}

// -----------------------------------------------------------------------
// draw the particles
// -----------------------------------------------------------------------

void particlesDraw(void) {

	float scale;
	int i;

	if (videoQuality == VIDEO_QUALITY_LOW)
		return;

	// LOD the particles
	if (gameCameraZ > -1100.0f)
		return;

	for (i = 0; i < PARTICLES_MAX; i++) {
		if (particles[i].type == PARTICLE_TYPE_NONE)
			continue;

		videoMatrixPush();

		videoTranslate3f(particles[i].pX, particles[i].pY, particles[i].pZ);

		switch (particles[i].type) {
		case PARTICLE_TYPE_DUST:
			scale = 0.1f - (2000.0f - particles[i].life)/20000.0f;
			videoScale3f(scale, scale, scale);
			break;

		case PARTICLE_TYPE_SMOKE_PIPE:
			scale = 0.1f - (4000.0f - particles[i].life)/40000.0f;
			videoScale3f(scale, scale, scale);
			break;
		}

		videoDrawMeshTextured(verticesBarrel, verticesBarrelTriangles);

		videoMatrixPop();
	}
}

// -----------------------------------------------------------------------
// the particle iteration
// -----------------------------------------------------------------------

void particlesIteration(float deltaTime) {

	int i;

	if (videoQuality == VIDEO_QUALITY_LOW)
		return;

	for (i = 0; i < PARTICLES_MAX; i++) {
		if (particles[i].type == PARTICLE_TYPE_NONE)
			continue;

		particles[i].life -= deltaTime;

		switch (particles[i].type) {
		case PARTICLE_TYPE_DUST:
			particles[i].pX += particles[i].speedX * deltaTime * 0.5f;
			particles[i].pY += particles[i].speedY * deltaTime * 0.5f;
			particles[i].pZ += particles[i].speedZ * deltaTime * 0.5f;
			break;

		case PARTICLE_TYPE_SMOKE_PIPE:
			particles[i].pZ += particles[i].speedZ * deltaTime;
			break;
		}

		if (particles[i].life <= 0.0f) {
			// the particle dies
			particles[i].type = PARTICLE_TYPE_NONE;
		}
	}
}

// -----------------------------------------------------------------------
// add a particle
// -----------------------------------------------------------------------

void particlesAdd(int type, float pX, float pY, float pZ, float speedX, float speedY, float speedZ) {

	int i;

	if (videoQuality == VIDEO_QUALITY_LOW)
		return;

	for (i = 0; i < PARTICLES_MAX; i++) {
		if (particles[i].type == PARTICLE_TYPE_NONE) {
			particles[i].type = type;
			particles[i].pX = pX;
			particles[i].pY = pY;
			particles[i].pZ = pZ;
			particles[i].speedX = speedX;
			particles[i].speedY = speedY;
			particles[i].speedZ = speedZ;
			particles[i].life = particleLifeTimes[type];
			return;
		}
	}
}
